package net.wanji.databus.event;

import com.alipay.jarslink.api.Module;
import com.alipay.jarslink.api.ModuleConfig;
import com.alipay.jarslink.api.ModuleLoader;
import com.alipay.jarslink.api.ModuleManager;
import net.wanji.common.framework.spring.ServiceBeanContext;
import net.wanji.common.utils.file.FastDfsUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.entity.develop.repo.RepoModule;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.cloud.client.ServiceInstance;
import org.springframework.cloud.client.loadbalancer.LoadBalancerClient;
import org.springframework.stereotype.Component;

import java.io.File;
import java.net.URL;
import java.util.List;

import static net.wanji.common.framework.Constants.applicationName.BP_DEVELOP;

/**
 * @ClassName RepoModuleEvent
 * @Description module 上下线事件
 * @Author wanji
 * @Date 2020/7/17 11:37
 * @Version 1.0
 */
@Component
@ConditionalOnClass(RepoModuleEvent.class) //判断该类在类路径下是否存在
@ConditionalOnProperty(value = "fdfs.fileDownUrl") //判断fastdfs的配置是否存在
@Slf4j
public class RepoModuleEvent {
    @Autowired
    private ModuleLoader moduleLoader;

    @Autowired
    private ModuleManager moduleManager;

    @Value("${fdfs.fileDownUrl}")
    public String fileDownUrl;

    @Autowired
    private FastDfsUtils fastDfsUtils;

    /**
     * @return void
     * @throws
     * @Author wanji
     * @Description nodule 下线
     * @Date 14:19 2020/5/28
     * @Param [repoModule]
     */
    public void moduleOfflineEvent(RepoModule repoModule) {
        try {
            if (repoModule == null) {
                return;
            }
            moduleManager.remove(repoModule.getName(), repoModule.getVersion());
            log.info("module 下线事件执行成功");
            if (StringUtils.isNotBlank(repoModule.getLocalPath())) {
                // 如果已经上线,也就是 jar 包已经下载到 本地,则将其删除
                if (fastDfsUtils.delLocalFile(repoModule.getLocalPath())) {
                    log.info("本地文件 : {} 删除成功", repoModule.getLocalPath());
                } else {
                    log.error("本地文件 : {} 删除失败", repoModule.getLocalPath());
                }
            }
        } catch (Exception e) {
            log.error("module 下线事件执行异常:{}", e);
        }
    }

    /**
     * @return void
     * @throws
     * @Author wanji
     * @Description nodule 上线
     * @Date 14:19 2020/5/28
     * @Param [repoModule]
     */
    public void moduleOnlineEvent(List<RepoModule> repoModuleList) {
        try {
            if (repoModuleList == null) {
                return;
            }
            repoModuleList.forEach(this::moduleOnline);
        } catch (Exception e) {
            log.error("module 上线事件执行异常:{}", e);
        }
    }

    private void moduleOnline(RepoModule repoModule) {
        try {
            String localPath = fileDownUrl + "/" +
                    repoModule.getName() + "/" +
                    repoModule.getVersion() + "/" +
                    repoModule.getFileName();
            boolean b = fastDfsUtils.downloadToLocal(localPath, repoModule.getPath());
            if (b) {
                // 装配模块配置文件
                URL file = new File(localPath).toURL();
                ModuleConfig moduleConfig = new ModuleConfig();
                moduleConfig.setName(repoModule.getName());
                moduleConfig.setEnabled(true);
                moduleConfig.setVersion(repoModule.getVersion());
                // develop : ip + port
                ServiceInstance serviceInstance = ServiceBeanContext.getBean(LoadBalancerClient.class).choose(BP_DEVELOP);
                String developIPAddress = "";
                if (serviceInstance != null && serviceInstance.getUri() != null) {
                    developIPAddress = serviceInstance.getUri().toString();
                }
                moduleConfig.setProperties(ImmutableMap.of("developIPAddress", developIPAddress));
                moduleConfig.setModuleUrl(ImmutableList.of(file));
                moduleConfig.addScanPackage(repoModule.getSpackage());
                // 加载模块
                Module load = moduleLoader.load(moduleConfig);
                // 注册模块
                moduleManager.register(load);
                log.info("moduleName:{} 上线成功", repoModule.getName());
            } else {
                log.error("文件下载到本地失败");
            }
        } catch (Exception e) {
            log.error("moduleName:{} 上线异常 , error:{}", repoModule.getName(), e);
        }
    }
}
