package net.wanji.databus.provider.integration.servicegateway.d2a;

import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.framework.rest.JsonViewObject;

import java.util.Map;

/**
 * @author wanji
 * 数据库基础接口定义
 */
public interface DBBaseProvider {

    /**
     * 单表：数据保存的基础接口。
     *
     * @param tableRestfulId tableRestfulId
     * @param runtimeParams  运行时的参数值。对应pluginJson中占位符的实际值。
     *                       比如：{
     *                       "name":"lzj",
     *                       "age":"11"
     *                       }
     * @return 执行结果。
     * @throws DubboProviderException 如果保存失败。未知的数据类型异常。未填写运行时参数。
     */
    JsonViewObject save(String tableRestfulId, Map<String, Object> runtimeParams) throws DubboProviderException;

    /**
     * 单表：数据修改的基础接口。
     * {
     * "set": {"name":"lzj","age":"11"},
     * "condition":{"id":"11"}
     * }
     *
     * @param tableRestfulId         tableRestfulId
     * @param partSetRuntimeParams   运行时的参数值。对应pluginJson中"set"部分占位符的实际值。
     *                               比如：{"name":"lzj","age":"11"}
     * @param partWhereRuntimeParams 运行时的参数值。 对应pluginJson中"condition"部分占位符的实际值。
     *                               比如：{"id":"11"}
     * @return 执行结果。
     * @throws DubboProviderException 如果更新失败。未知的数据类型异常。未填写运行时参数。
     */
    JsonViewObject update(String tableRestfulId, Map<String, Object> partSetRuntimeParams,
                          Map<String, Object> partWhereRuntimeParams) throws DubboProviderException;

    /**
     * 单表：数据删除的基础接口
     *
     * @param tableRestfulId tableRestfulId
     * @param runtimeParams  运行时的参数值。对应pluginJson中占位符的实际值。
     * @return 执行结果。
     * @throws DubboProviderException 如果删除失败。未知的数据类型异常。未填写运行时参数。
     */
    JsonViewObject delete(String tableRestfulId, Map<String, Object> runtimeParams) throws DubboProviderException;

    /**
     * 单表：单条数据查询接口。
     *
     * @param tableRestfulId tableRestfulId
     * @param runtimeParams  运行时的参数值。对应pluginJson中占位符的实际值。
     * @return 唯一主键对应的记录。
     * @throws DubboProviderException 如果查询失败。未知的数据类型异常。未填写运行时参数。
     */
    JsonViewObject findById(String tableRestfulId, Map<String, Object> runtimeParams) throws DubboProviderException;

    /**
     * 单表：查询符合条件的所有的数据接口。
     *
     * @param tableRestfulId tableRestfulId
     * @param runtimeParams  运行时的参数值。对应pluginJson中占位符的实际值。
     * @return 符合条件的记录。
     * @throws DubboProviderException 如果查询失败。未知的数据类型异常。未填写运行时参数。
     */
    JsonViewObject findByWhere(String tableRestfulId, Map<String, Object> runtimeParams) throws DubboProviderException;

    /**
     * 单表：分页查询接口。
     *
     * @param tableRestfulId tableRestfulId
     * @param runtimeParams  运行时的参数值。pluginJson中占位符对应的实际值
     *                       比如：{
     *                       "condition":{"name":"lzj","age":"11"},
     *                       "pageSize":"1",
     *                       "pageNum":"2"
     *                       }
     * @return 符合条件的分页记录。
     * @throws DubboProviderException 如果分页查询失败。未知的数据类型异常。未填写运行时参数。
     */
    JsonViewObject findByPage(String tableRestfulId, Map<String, Object> runtimeParams) throws DubboProviderException;

    /**
     * 多表查询。
     * 分页和条件查询都走这一个接口。所有的占位符参数都在condition中保存。
     * 后台直接将参数使用JUEL替换sql中占位符，执行即可。
     *
     * @param tabRestfulId  tableRestfulId
     * @param runtimeParams 运行时的参数值。对应pluginJson中占位符的实际值。
     *                      比如：
     *                      {
     *                      "condition": [
     *                      {
     *                      "fieldNameEn": "S_STATUS",
     *                      "type": "VARCHAR",
     *                      "value": ""
     *                      "des": ""
     *                      }
     *                      ]
     *                      }
     * @return 符合条件的记录。
     * @throws DubboProviderException 如果查询失败。未知的数据类型异常。未填写运行时参数。
     */
    JsonViewObject bySQL(String tabRestfulId, Map<String, Object> runtimeParams) throws DubboProviderException;
}

